const GB_DRAG_SVC = Components.classes["@mozilla.org/widget/dragservice;1"].getService(Components.interfaces.nsIDragService);

const GB_BTN_PARENT_ID = "ggb_toolbar_buttons";
const GB_FAKE_SEP = "ggb_separator_fake";

const GB_DRAG_DEF_ORDER = [
	"ggb_CustMenu", "ggb_search_separate", "ggb_site_separate", "ggb_lucky_separate", "ggb_images_separate",
	"ggb_video_separate", "ggb_groups_separate", "ggb_directory_separate", "ggb_news_separate", "ggb_answers_separate",
	"ggb_scholar_separate", "ggb_print_separate", "ggb_desktop_separate", "ggb_catalog_separate", "ggb_froogle_separate",
	"ggb_stocks_separate", "ggb_unclesam_separate", "ggb_dictionary_separate", "ggb_glossary_separate", "ggb_nocalc_separate",
	"ggb_blogger_separate", "ggb_maps_separate", "ggb_mycountry_separate", "ggb_mylocal_separate", "ggb_bsd_separate",
	"ggb_linux_separate", "ggb_mac_separate", "ggb_microsoft_separate", "ggb_separator_01", "ggb_options_separate",
	"ggb_gmail_separate", "ggb_SrchMenu", "ggb_SpecMenu", "ggb_CompMenu", "ggb_UnivButton",
	"ggb_separator_02", "ggb_extras_box", "ggb_InfoMenu", "ggb_cached_separate", "ggb_similar_separate",
	"ggb_backwards_separate", "ggb_translate_separate", "ggb_UpDirButton", "ggb_navigation_box", "ggb_separator_03",
	"ggb_highlightplus_button", "ggb_highlight_button"
];

/* Drag and Drop Observers */

const GB_DRAG_OBSVR_OFF = {
};

const GB_DRAG_OBSVR_ON = {

	mCreator: null,

	getSupportedFlavours: function () {
		var flavours = new FlavourSet();
		flavours.appendFlavour("text/unicode");
		return flavours;
	},

	onDragExit: function (evt, session) {
		myGooglebarUtil.updateClass(evt.target, 1, "ggb_css_drag_border_none");
	},

	onDragOver: function (evt, flavour, session) {
		if (this.mCreator == evt.target.getAttribute("id") || this.mCreator == "invalid") {
			return;
		}

		var halfX = evt.target.boxObject.x + (evt.target.boxObject.width / 2);

		if (evt.pageX < halfX) {
			myGooglebarUtil.updateClass(evt.target, 1, "ggb_css_dragborder_left");
		}
		else {
			myGooglebarUtil.updateClass(evt.target, 1, "ggb_css_drag_border_right");
		}
	},

	onDragStart: function (evt, transferData, action) {
		if (evt.target.tagName == "menuitem") {
			this.mCreator = "invalid";
			return;
		}

		var txt = evt.target.getAttribute("id");
		this.mCreator = txt;
		transferData.data = new TransferData();
		transferData.data.addDataForFlavour("text/unicode", txt);

		if ((evt.target.getAttribute("type") == "menu" || evt.target.getAttribute("type") == "menu-button") && evt.target.firstChild) {
			evt.target.firstChild.hidePopup();
		}
	},

	onDrop: function (evt, dropdata, session) {
		if (dropdata.data != "") {
			this.mCreator = null;
			var parElem = evt.target.parentNode;
			var btmElem = evt.target;
			var topElem = myGooglebarUtil.getElement(dropdata.data);

			var classes = btmElem.getAttribute("class").split(" ");
			var secondaryClass = classes[1];

			if (secondaryClass == "ggb_css_dragborder_left")
				myGooglebarDnD.insertBefore(topElem.getAttribute("id"), btmElem.getAttribute("id"));
			else if (secondaryClass == "ggb_css_drag_border_right")
				myGooglebarDnD.insertAfter(topElem.getAttribute("id"), btmElem.getAttribute("id"));
			else
				alert("error: drop occurred on invalid target");

			myGooglebarDnD.updateOrdinals();
		}

		myGooglebarUtil.updateClass(evt.target, 1, "ggb_css_drag_border_none");
	}
};

var myGooglebarDnD = {

	mCustomOrder: null,
	mDragObsvr: GB_DRAG_OBSVR_ON,

	getNextVisibleElement: function(aStart) {
		var element;
	
		for (var i = aStart; i < this.mCustomOrder.length; i++) {
			element = myGooglebarUtil.getElement(this.mCustomOrder[i]);
	
			if (element.getAttribute("hidden") == "false") {
				return this.mCustomOrder[i];
			}
		}
	
		/* if no visible elements found then force the separator to be hidden */
		return GB_FAKE_SEP;
	},

	initCustomOrder: function() {
		var element;
		var parent = myGooglebarUtil.getElement(GB_BTN_PARENT_ID);
	
		this.mCustomOrder = new Array(parent.childNodes.length);
	
		for (var i = 0; i < parent.childNodes.length; i++) {
			element = parent.childNodes[i];
			this.mCustomOrder[element.getAttribute("ordinal") - 1] = element.getAttribute("id");
		}
	},

	insertAfter: function(aTopElemID, aBtmElemID) {
		this.insertItem(aTopElemID, aBtmElemID, false);
	},
	
	insertBefore: function(aTopElemID, aBtmElemID) {
		this.insertItem(aTopElemID, aBtmElemID, true);
	},
	
	insertItem: function(aTopElemID, aBtmElemID, aPutBefore) {
		var topElemIdx = -1;
		var btmElemIdx = -1;
		var i;
	
		for (i = 0; i < this.mCustomOrder.length; i++) {
			if (this.mCustomOrder[i] == aTopElemID) {
				topElemIdx = i;
				break;
			}
		}
	
		if (topElemIdx < 0)
			alert("error: top element ID not found in collection");
	
		var removed = this.mCustomOrder.splice(topElemIdx, 1);
	
		for (i = 0; i < this.mCustomOrder.length; i++) {
			if (this.mCustomOrder[i] == aBtmElemID) {
				btmElemIdx = i;
				break;
			}
		}
	
		if (btmElemIdx < 0)
			alert("error: btm element ID not found in collection");
	
		if (aPutBefore)
			this.mCustomOrder.splice(btmElemIdx, 1, aTopElemID, aBtmElemID);
		else
			this.mCustomOrder.splice(btmElemIdx, 1, aBtmElemID, aTopElemID);
	},
	
	lockOrdinalOrder: function() {
		this.mDragObsvr = GB_DRAG_OBSVR_OFF;
	},
	
	unlockOrdinalOrder: function() {
		this.mDragObsvr = GB_DRAG_OBSVR_ON;
	},
	
	resetOrdinals: function() {
		for (var i = 0; i < GB_DRAG_DEF_ORDER.length; i++) {
			this.mCustomOrder[i] = GB_DRAG_DEF_ORDER[i];
		}

		this.updateOrdinals();
	},
	
	updateOrdinals: function() {
		if (!this.mCustomOrder)
			this.initCustomOrder();
	
		var element;
		var parent = myGooglebarUtil.getElement(GB_BTN_PARENT_ID);
	
		for (var i = 0; i < this.mCustomOrder.length; i++) {
			element = myGooglebarUtil.getElement(this.mCustomOrder[i]);
	    
			if (!element) {
				alert('error: Could not find element (' + i + ') ' + this.mCustomOrder[i] + ' in current document');
			}
			else {
				element.setAttribute("ordinal", i + 1);
			}
		}
	
		this.updateSeparators();
	
		// Need a quick little refresh due to the apparent Mozilla problem with
		// rendering menu buttons in the correct order after an ordinal change
		parent.setAttribute("hidden", "true");
		parent.setAttribute("hidden", "false");
	},
	
	updateSeparators: function() {
		if (!this.mCustomOrder)
			this.initCustomOrder();
	
	
		var element, next;
	
		/* if a separator is next to another separator or the nav box then hide it */
		for (var i = 0; i < this.mCustomOrder.length - 1; i++) {
			if (/.*separator.*/.test(this.mCustomOrder[i])) {
				element = myGooglebarUtil.getElement(this.mCustomOrder[i]);
				
				if (!myGooglebarPrefUtil.getPrefValue(GB_PREF_BUTTON_SEPARATORS_NAME)) {
					element.setAttribute("hidden", "true");
				}
				else {
					next = this.getNextVisibleElement(i+1);
		      
					if ((/.*separator.*/.test(next) || next == "ggb_navigation_box")) {
						element.setAttribute("hidden", "true");
					}
					else {
						element.setAttribute("hidden", "false");
					}
				}
			}
		}
	
		/* if the last item is a separator then hide it */
		if (/.*separator.*/.test(this.mCustomOrder[this.mCustomOrder.length - 1])) {
			element = myGooglebarUtil.getElement(this.mCustomOrder[this.mCustomOrder.length - 1]);
			element.setAttribute("hidden", "true");
		}
	}
};
