/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Googlebar for Mozilla.
 *
 * The Initial Developer of the Original Code is Andy Edmonds.
 * Portions created by the Initial Developer are Copyright (C) 2001
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Andy Boughton
 *  Andrew Houghton
 *  John Woods
 *  Bernd Kuemmerlen
 *  Robert Mulcahy
 *  Alfred Kayser
 *  Reflex
 *  Gary Turnbull
 *  Raj Bhaskar
 *  Joachim Thewes
 *  Henrik Gemal
 *  Robert Fernandes
 *  Joe Chellman
 *  Franki Cheung
 *  Alban Fonrouge
 *  Martin Hassman
 *  Ufuk Kayserilioglu
 *  Yoni Gilad
 *  Tim Schmidt
 *  timeless
 *  Francis Turner
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/***********************************
   Preference Types
 ***********************************/

const GB_PREF_TYPE_ARRAY = "array";
const GB_PREF_TYPE_BOOL  = "bool";
const GB_PREF_TYPE_CHAR  = "char";
const GB_PREF_TYPE_INT   = "int";

/***********************************
   Services, Objects, etc.
 ***********************************/

try {
	const GB_STRING_FLAG = "nsISupportsWString" in Components.interfaces && Components.interfaces.nsISupportsWString || "nsISupportsString" in Components.interfaces && Components.interfaces.nsISupportsString;
	const GB_STRING_CONSTRUCTOR = Components.Constructor("@mozilla.org/supports-wstring;1" in Components.classes && Components.classes["@mozilla.org/supports-wstring;1"] || "@mozilla.org/supports-string;1" in Components.classes && Components.classes["@mozilla.org/supports-string;1"], GB_STRING_FLAG);
}
catch (e) {
	myGooglebarUtil.logMessage("Failed to initialize string constructor\n" + e);
}

try {
	const GB_PREF_SVC   = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
}
catch (e) {
	myGooglebarUtil.logMessage("Failed to initialize pref service\n" + e);
}

const GB_ROOT_OBJ = GB_PREF_SVC.getBranch(null);
const GB_PREF_OBJ = GB_PREF_SVC.getBranch("googlebar.");
const GB_PREF_SEP = "||";

/***********************************
   Modifier Key Action Preferences
 ***********************************/
	     
const GB_PREF_ACTION_ALT_NAME  = "ActionAlt";
const GB_PREF_ACTION_ALT_VALUE = "0";

const GB_PREF_ACTION_ALT_CTRL_NAME  = "ActionAltCtrl";
const GB_PREF_ACTION_ALT_CTRL_VALUE = "0";

const GB_PREF_ACTION_ALT_SHIFT_NAME  = "ActionAltShift";
const GB_PREF_ACTION_ALT_SHIFT_VALUE = "0";

const GB_PREF_ACTION_CTRL_NAME  = "ActionCtrl";
const GB_PREF_ACTION_CTRL_VALUE = "0";

const GB_PREF_ACTION_CTRL_SHIFT_NAME  = "ActionCtrlShift";
const GB_PREF_ACTION_CTRL_SHIFT_VALUE = "0";

const GB_PREF_ACTION_NONE_NAME  = "ActionNone";
const GB_PREF_ACTION_NONE_VALUE = "0";

const GB_PREF_ACTION_SHIFT_NAME  = "ActionShift";
const GB_PREF_ACTION_SHIFT_VALUE = "0";

/*************************************
   Modifier Key Location Preferences
 *************************************/

const GB_PREF_LOC_ALT_NAME  = "LocationAlt";
const GB_PREF_LOC_ALT_VALUE = "0";

const GB_PREF_LOC_ALT_CTRL_NAME  = "LocationAltCtrl";
const GB_PREF_LOC_ALT_CTRL_VALUE = "0";

const GB_PREF_LOC_ALT_SHIFT_NAME  = "LocationAltShift";
const GB_PREF_LOC_ALT_SHIFT_VALUE = "0";

const GB_PREF_LOC_CTRL_NAME  = "LocationCtrl";
const GB_PREF_LOC_CTRL_VALUE = "1";

const GB_PREF_LOC_CTRL_SHIFT_NAME  = "LocationCtrlShift";
const GB_PREF_LOC_CTRL_SHIFT_VALUE = "1";

const GB_PREF_LOC_NONE_NAME  = "LocationNone";
const GB_PREF_LOC_NONE_VALUE = "0";

const GB_PREF_LOC_SHIFT_NAME  = "LocationShift";
const GB_PREF_LOC_SHIFT_VALUE = "2";

/****************************************
   Domain Specific Preferences
 ****************************************/

const GB_PREF_COUNTRY_SEARH_NAME = "country2Search";
const GB_PREF_COUNTRY_SEARH_VALUE = 80;
	
const GB_PREF_MY_COUNTRY_SEARH_NAME = "mycountry2Search";
const GB_PREF_MY_COUNTRY_SEARH_VALUE = 0;

const GB_PREF_MY_LANGUAGE_SEARH_NAME = "mylang2Search";
const GB_PREF_MY_LANGUAGE_SEARH_VALUE = 0;

const GB_PREF_COUNTRY_NEWS_SEARH_NAME = "countryNews2Search";
const GB_PREF_COUNTRY_NEWS_SEARH_VALUE = 0;

const GB_PREF_COUNTRY_MAPS_SEARH_NAME = "countryMaps2Search";
const GB_PREF_COUNTRY_MAPS_SEARH_VALUE = 0;

const GB_PREF_COUNTRY_FROOGLE_SEARH_NAME = "countryFroogle2Search";
const GB_PREF_COUNTRY_FROOGLE_SEARH_VALUE = 0;

/***********************************
   Other Miscellaneous Preferences
 ***********************************/

const GB_PREF_AUTO_SEARCH_NAME = "autosearchoption";
const GB_PREF_AUTO_SEARCH_VALUE = false;

const GB_PREF_CONTEXT_MENU_NAME = "contextmenuoption";
const GB_PREF_CONTEXT_MENU_VALUE = true;

const GB_PREF_HIDE_MENU_NAME = "hidemenuoption";
const GB_PREF_HIDE_MENU_VALUE = false;

const GB_PREF_HISTORY_SAVE_NAME = "historyoption";
const GB_PREF_HISTORY_SAVE_VALUE = true;

const GB_PREF_HISTORY_TERMS_NAME = "history";
const GB_PREF_HISTORY_TERMS_VALUE = new Array(0);

const GB_PREF_HISTORY_DICT_NAME = "dictHistory";
const GB_PREF_HISTORY_DICT_VALUE = true;

const GB_PREF_KEY_SELECTION_NAME = "hotkeySelectionToggles";
const GB_PREF_KEY_SELECTION_VALUE = false;

const GB_PREF_SELECTION_APPENDS_NAME = "search4SelectedAppends";
const GB_PREF_SELECTION_APPENDS_VALUE = false;

const GB_PREF_HISTORY_MAX_NAME = "maxHistCnt";
const GB_PREF_HISTORY_MAX_VALUE = 10;

const GB_PREF_UP_IGNORE_QUERY_NAME = "querystringoption";
const GB_PREF_UP_IGNORE_QUERY_VALUE = false;

const GB_PREF_AUTO_SELECT_NAME = "autoselectoption";
const GB_PREF_AUTO_SELECT_VALUE = true;

const GB_PREF_SAVE_LAST_SEARCH_NAME = "savelastoption";
const GB_PREF_SAVE_LAST_SEARCH_VALUE = false;

const GB_PREF_RESULT_DATE_SORT_NAME = "sortByDate";
const GB_PREF_RESULT_DATE_SORT_VALUE = false;

const GB_PREF_RESULT_MAX_NAME = "resultLimit";
const GB_PREF_RESULT_MAX_VALUE = -1;

const GB_PREF_UNIVERSITY_ID_NAME = "university";
const GB_PREF_UNIVERSITY_ID_VALUE = "";

const GB_PREF_UNIVERSITY_LABEL_NAME = "universityName";
const GB_PREF_UNIVERSITY_LABEL_VALUE = "";

const GB_PREF_BUTTON_LABELS_NAME = "showLabels";
const GB_PREF_BUTTON_LABELS_VALUE = false;

const GB_PREF_BUTTON_SEPARATORS_NAME = "showSeparators";
const GB_PREF_BUTTON_SEPARATORS_VALUE = true;

const GB_PREF_LOCK_ORDINALS_NAME = "lockOrdinals";
const GB_PREF_LOCK_ORDINALS_VALUE = true;

const GB_PREF_CUSTOM_LIST_NAME = "customList";
const GB_PREF_CUSTOM_LIST_VALUE = new Array(0);

const GB_PREF_MY_LOCAL_NAME = "gbmylocalPref";
const GB_PREF_MY_LOCAL_VALUE = "";

const GB_PREF_GLOBAL_CRITERIA_NAME = "globalCriteria";
const GB_PREF_GLOBAL_CRITERIA_VALUE = "";

const GB_PREF_TERM_COLORS_NAME = "termColors";
const GB_PREF_TERM_COLORS_VALUE = [ "yellow", "lightpink", "aquamarine", "darkgoldenrod", "darkseagreen", "lightgreen", "rosybrown", "seagreen", "chocolate", "violet" ];

const GB_PREF_THUMBNAILS_NAME = "thumbnails";
const GB_PREF_THUMBNAILS_VALUE = false;

const GB_PREF_AUTO_HIGHLIGHT_NAME = "autoHighlight";
const GB_PREF_AUTO_HIGHLIGHT_VALUE = false;

const GB_PREF_FORM_HIGHLIGHT_NAME = "formHighlight";
const GB_PREF_FORM_HIGHLIGHT_VALUE = true;

const GB_PREF_WEB_NAVIGATION_NAME = "webNavigation";
const GB_PREF_WEB_NAVIGATION_VALUE = true;

const GB_PREF_SATELLITE_NAME = "satellite";
const GB_PREF_SATELLITE_VALUE = false;

const GB_PREF_PAGE_SELECTION_NAME = "pageSelection";
const GB_PREF_PAGE_SELECTION_VALUE = false;

/*************************************
   Experimental Preferences
 *************************************/

const GB_PREF_GOOGLE_SUGGEST_NAME = "googleSuggest";
const GB_PREF_GOOGLE_SUGGEST_VALUE = false;

const GB_PREF_GOOGLE_SUGGEST_FREQ_NAME = "googleSuggestFreq";
const GB_PREF_GOOGLE_SUGGEST_FREQ_VALUE = 0;

const GB_PREF_GOOGLE_LOGIN_NAME = "googleLogin";
const GB_PREF_GOOGLE_LOGIN_VALUE = false;

const GB_PREF_GMAIL_USERID_NAME = "gmailUserID";
const GB_PREF_GMAIL_USERID_VALUE = "";

const GB_PREF_GMAIL_UPDATE_NAME = "gmailUpdate";
const GB_PREF_GMAIL_UPDATE_VALUE = false;

const GB_PREF_GMAIL_DELAY_NAME = "gmailDelay";
const GB_PREF_GMAIL_DELAY_VALUE = 5;

const GB_PREF_GMAIL_NOTIFY_ENABLED_NAME = "gmailNotifyEnabled";
const GB_PREF_GMAIL_NOTIFY_ENABLED_VALUE = true;

const GB_PREF_GMAIL_NOTIFY_NAME = "gmailNotify";
const GB_PREF_GMAIL_NOTIFY_VALUE = "";

const GB_PREF_DESKTOP_URL_NAME = "desktopURL";
const GB_PREF_DESKTOP_URL_VALUE = "http://127.0.0.1:4664/";

const GB_PREF_DESKTOP_ID_NAME = "desktopID";
const GB_PREF_DESKTOP_ID_VALUE = "";

const GB_PREF_DESKTOP_SEARCH_ID_NAME = "desktopSearchID";
const GB_PREF_DESKTOP_SEARCH_ID_VALUE = "";

/*************************************
   Hidden Preferences
 *************************************/

const GB_PREF_BLOG_STD_WIN_NAME = "bloggerStandardWinPref";
const GB_PREF_BLOG_STD_WIN_VALUE = true;

const GB_PREF_LAST_SEARCH_COMP_NAME = "lastCompSearch";
const GB_PREF_LAST_SEARCH_COMP_VALUE = "";

const GB_PREF_LAST_SEARCH_CUST_NAME = "lastCustSearch";
const GB_PREF_LAST_SEARCH_CUST_VALUE = "";

const GB_PREF_LAST_SEARCH_INFO_NAME = "lastInfoSearch";
const GB_PREF_LAST_SEARCH_INFO_VALUE = "";

const GB_PREF_LAST_SEARCH_SRCH_NAME = "lastSrchSearch";
const GB_PREF_LAST_SEARCH_SRCH_VALUE = "";

const GB_PREF_LAST_SEARCH_SPEC_NAME = "lastSpecSearch";
const GB_PREF_LAST_SEARCH_SPEC_VALUE = "";

const GB_PREF_GMAIL_URL_NAME = "gmailHome";
const GB_PREF_GMAIL_URL_VALUE = "https://mail.google.com/mail";
// const GB_PREF_GMAIL_URL_VALUE = "https://mail.google.com/mail/h/";


/*************************************
   Preference Utility Object
 *************************************/

var myGooglebarPrefUtil = {

	mPrefsInit: false,

	mTypesTable: new Array(),
 
	initialize: function() {
	    /***********************************
	       Modifier Key Action Preferences
	     ***********************************/

		this.initPref(GB_PREF_ACTION_ALT_NAME, GB_PREF_ACTION_ALT_VALUE, GB_PREF_TYPE_CHAR);
		this.initPref(GB_PREF_ACTION_ALT_CTRL_NAME, GB_PREF_ACTION_ALT_CTRL_VALUE, GB_PREF_TYPE_CHAR);
		this.initPref(GB_PREF_ACTION_ALT_SHIFT_NAME, GB_PREF_ACTION_ALT_SHIFT_VALUE, GB_PREF_TYPE_CHAR);
		this.initPref(GB_PREF_ACTION_CTRL_NAME, GB_PREF_ACTION_CTRL_VALUE, GB_PREF_TYPE_CHAR);
		this.initPref(GB_PREF_ACTION_CTRL_SHIFT_NAME, GB_PREF_ACTION_CTRL_SHIFT_VALUE, GB_PREF_TYPE_CHAR);
		this.initPref(GB_PREF_ACTION_NONE_NAME, GB_PREF_ACTION_NONE_VALUE, GB_PREF_TYPE_CHAR);
		this.initPref(GB_PREF_ACTION_SHIFT_NAME, GB_PREF_ACTION_SHIFT_VALUE, GB_PREF_TYPE_CHAR);
	        
	    /*************************************
	       Modifier Key Location Preferences
	     *************************************/
	
		this.initPref(GB_PREF_LOC_ALT_NAME, GB_PREF_LOC_ALT_VALUE, GB_PREF_TYPE_CHAR);
		this.initPref(GB_PREF_LOC_ALT_CTRL_NAME, GB_PREF_LOC_ALT_CTRL_VALUE, GB_PREF_TYPE_CHAR);
		this.initPref(GB_PREF_LOC_ALT_SHIFT_NAME, GB_PREF_LOC_ALT_SHIFT_VALUE, GB_PREF_TYPE_CHAR);
		this.initPref(GB_PREF_LOC_CTRL_NAME, GB_PREF_LOC_CTRL_VALUE, GB_PREF_TYPE_CHAR);
		this.initPref(GB_PREF_LOC_CTRL_SHIFT_NAME, GB_PREF_LOC_CTRL_SHIFT_VALUE, GB_PREF_TYPE_CHAR);
		this.initPref(GB_PREF_LOC_NONE_NAME, GB_PREF_LOC_NONE_VALUE, GB_PREF_TYPE_CHAR);
		this.initPref(GB_PREF_LOC_SHIFT_NAME, GB_PREF_LOC_SHIFT_VALUE, GB_PREF_TYPE_CHAR);
				
		/****************************************
	       Domain Specific Preferences (Search)
	     ****************************************/

		this.initPref(GB_PREF_COUNTRY_SEARH_NAME, GB_PREF_COUNTRY_SEARH_VALUE, GB_PREF_TYPE_INT);
		this.initPref(GB_PREF_MY_COUNTRY_SEARH_NAME, GB_PREF_MY_COUNTRY_SEARH_VALUE, GB_PREF_TYPE_INT);
		this.initPref(GB_PREF_MY_LANGUAGE_SEARH_NAME, GB_PREF_MY_LANGUAGE_SEARH_VALUE, GB_PREF_TYPE_INT);
		this.initPref(GB_PREF_COUNTRY_NEWS_SEARH_NAME, GB_PREF_COUNTRY_NEWS_SEARH_VALUE, GB_PREF_TYPE_INT);
		this.initPref(GB_PREF_COUNTRY_MAPS_SEARH_NAME, GB_PREF_COUNTRY_MAPS_SEARH_VALUE, GB_PREF_TYPE_INT);
		this.initPref(GB_PREF_COUNTRY_FROOGLE_SEARH_NAME, GB_PREF_COUNTRY_FROOGLE_SEARH_VALUE, GB_PREF_TYPE_INT);

	    /***********************************
	       Other Miscellaneous Preferences
	     ***********************************/

		this.initPref(GB_PREF_AUTO_SEARCH_NAME, GB_PREF_AUTO_SEARCH_VALUE, GB_PREF_TYPE_BOOL);
		this.initPref(GB_PREF_CONTEXT_MENU_NAME, GB_PREF_CONTEXT_MENU_VALUE, GB_PREF_TYPE_BOOL);
		this.initPref(GB_PREF_HIDE_MENU_NAME, GB_PREF_HIDE_MENU_VALUE, GB_PREF_TYPE_BOOL);
		this.initPref(GB_PREF_HISTORY_SAVE_NAME, GB_PREF_HISTORY_SAVE_VALUE, GB_PREF_TYPE_BOOL);
		this.initPref(GB_PREF_HISTORY_TERMS_NAME, GB_PREF_HISTORY_TERMS_VALUE, GB_PREF_TYPE_ARRAY);
		this.initPref(GB_PREF_HISTORY_DICT_NAME, GB_PREF_HISTORY_DICT_VALUE, GB_PREF_TYPE_BOOL);
		this.initPref(GB_PREF_KEY_SELECTION_NAME, GB_PREF_KEY_SELECTION_VALUE, GB_PREF_TYPE_BOOL);
		this.initPref(GB_PREF_SELECTION_APPENDS_NAME, GB_PREF_SELECTION_APPENDS_VALUE, GB_PREF_TYPE_BOOL);
		this.initPref(GB_PREF_HISTORY_MAX_NAME, GB_PREF_HISTORY_MAX_VALUE, GB_PREF_TYPE_INT);
		this.initPref(GB_PREF_UP_IGNORE_QUERY_NAME, GB_PREF_UP_IGNORE_QUERY_VALUE, GB_PREF_TYPE_BOOL);
		this.initPref(GB_PREF_AUTO_SELECT_NAME, GB_PREF_AUTO_SELECT_VALUE, GB_PREF_TYPE_BOOL);
		this.initPref(GB_PREF_SAVE_LAST_SEARCH_NAME, GB_PREF_SAVE_LAST_SEARCH_VALUE, GB_PREF_TYPE_BOOL);
		this.initPref(GB_PREF_RESULT_DATE_SORT_NAME, GB_PREF_RESULT_DATE_SORT_VALUE, GB_PREF_TYPE_BOOL);
		this.initPref(GB_PREF_RESULT_MAX_NAME, GB_PREF_RESULT_MAX_VALUE, GB_PREF_TYPE_INT);
		this.initPref(GB_PREF_UNIVERSITY_ID_NAME, GB_PREF_UNIVERSITY_ID_VALUE, GB_PREF_TYPE_CHAR);
		this.initPref(GB_PREF_UNIVERSITY_LABEL_NAME, GB_PREF_UNIVERSITY_LABEL_VALUE, GB_PREF_TYPE_CHAR);
		this.initPref(GB_PREF_BUTTON_LABELS_NAME, GB_PREF_BUTTON_LABELS_VALUE, GB_PREF_TYPE_BOOL);
		this.initPref(GB_PREF_BUTTON_SEPARATORS_NAME, GB_PREF_BUTTON_SEPARATORS_VALUE, GB_PREF_TYPE_BOOL);
		this.initPref(GB_PREF_LOCK_ORDINALS_NAME, GB_PREF_LOCK_ORDINALS_VALUE, GB_PREF_TYPE_BOOL);
		this.initPref(GB_PREF_CUSTOM_LIST_NAME, GB_PREF_CUSTOM_LIST_VALUE, GB_PREF_TYPE_ARRAY);
		this.initPref(GB_PREF_MY_LOCAL_NAME, GB_PREF_MY_LOCAL_VALUE, GB_PREF_TYPE_CHAR);
		this.initPref(GB_PREF_GLOBAL_CRITERIA_NAME, GB_PREF_GLOBAL_CRITERIA_VALUE, GB_PREF_TYPE_CHAR);
		this.initPref(GB_PREF_TERM_COLORS_NAME, GB_PREF_TERM_COLORS_VALUE, GB_PREF_TYPE_ARRAY);
		this.initPref(GB_PREF_THUMBNAILS_NAME, GB_PREF_THUMBNAILS_VALUE, GB_PREF_TYPE_BOOL);
		this.initPref(GB_PREF_AUTO_HIGHLIGHT_NAME, GB_PREF_AUTO_HIGHLIGHT_VALUE, GB_PREF_TYPE_BOOL);
		this.initPref(GB_PREF_FORM_HIGHLIGHT_NAME, GB_PREF_FORM_HIGHLIGHT_VALUE, GB_PREF_TYPE_BOOL);
		this.initPref(GB_PREF_WEB_NAVIGATION_NAME, GB_PREF_WEB_NAVIGATION_VALUE, GB_PREF_TYPE_BOOL);
		this.initPref(GB_PREF_SATELLITE_NAME, GB_PREF_SATELLITE_VALUE, GB_PREF_TYPE_BOOL);
		this.initPref(GB_PREF_PAGE_SELECTION_NAME, GB_PREF_PAGE_SELECTION_VALUE, GB_PREF_TYPE_BOOL);
	
	    /*************************************
	       Experimental Preferences
	     *************************************/

		this.initPref(GB_PREF_GOOGLE_SUGGEST_NAME, GB_PREF_GOOGLE_SUGGEST_VALUE, GB_PREF_TYPE_BOOL);
		this.initPref(GB_PREF_GOOGLE_SUGGEST_FREQ_NAME, GB_PREF_GOOGLE_SUGGEST_FREQ_VALUE, GB_PREF_TYPE_INT);
		this.initPref(GB_PREF_GOOGLE_LOGIN_NAME, GB_PREF_GOOGLE_LOGIN_VALUE, GB_PREF_TYPE_BOOL);
		this.initPref(GB_PREF_GMAIL_USERID_NAME, GB_PREF_GMAIL_USERID_VALUE, GB_PREF_TYPE_CHAR);
		this.initPref(GB_PREF_GMAIL_UPDATE_NAME, GB_PREF_GMAIL_UPDATE_VALUE, GB_PREF_TYPE_BOOL);
		this.initPref(GB_PREF_GMAIL_DELAY_NAME, GB_PREF_GMAIL_DELAY_VALUE, GB_PREF_TYPE_INT);
		this.initPref(GB_PREF_GMAIL_NOTIFY_ENABLED_NAME, GB_PREF_GMAIL_NOTIFY_ENABLED_VALUE, GB_PREF_TYPE_BOOL);
		this.initPref(GB_PREF_GMAIL_NOTIFY_NAME, GB_PREF_GMAIL_NOTIFY_VALUE, GB_PREF_TYPE_CHAR);
		this.initPref(GB_PREF_DESKTOP_URL_NAME, GB_PREF_DESKTOP_URL_VALUE, GB_PREF_TYPE_CHAR);
		this.initPref(GB_PREF_DESKTOP_ID_NAME, GB_PREF_DESKTOP_ID_VALUE, GB_PREF_TYPE_CHAR);
		this.initPref(GB_PREF_DESKTOP_SEARCH_ID_NAME, GB_PREF_DESKTOP_SEARCH_ID_VALUE, GB_PREF_TYPE_CHAR);
		
		/*************************************
		   Hidden Preferences
		 *************************************/

		this.initPref(GB_PREF_BLOG_STD_WIN_NAME, GB_PREF_BLOG_STD_WIN_VALUE, GB_PREF_TYPE_BOOL);
		this.initPref(GB_PREF_LAST_SEARCH_COMP_NAME, GB_PREF_LAST_SEARCH_COMP_VALUE, GB_PREF_TYPE_CHAR);
		this.initPref(GB_PREF_LAST_SEARCH_CUST_NAME, GB_PREF_LAST_SEARCH_CUST_VALUE, GB_PREF_TYPE_CHAR);
		this.initPref(GB_PREF_LAST_SEARCH_INFO_NAME, GB_PREF_LAST_SEARCH_INFO_VALUE, GB_PREF_TYPE_CHAR);
		this.initPref(GB_PREF_LAST_SEARCH_SPEC_NAME, GB_PREF_LAST_SEARCH_SPEC_VALUE, GB_PREF_TYPE_CHAR);
		this.initPref(GB_PREF_LAST_SEARCH_SRCH_NAME, GB_PREF_LAST_SEARCH_SRCH_VALUE, GB_PREF_TYPE_CHAR);
		this.initPref(GB_PREF_GMAIL_URL_NAME, GB_PREF_GMAIL_URL_VALUE, GB_PREF_TYPE_CHAR);
		
		this.mPrefInit = true;
	},
	
	initPref: function(aPrefName, aPrefValue, aPrefType) {
		if (aPrefName == aPrefValue) {
			myGooglebarUtil.logMessage("Pref name and value are indentical (" + aPrefName + "), this is likely an initialization error");
			return;
		}
	
		switch (aPrefType) {

			case GB_PREF_TYPE_ARRAY:
				try {
					GB_PREF_OBJ.getComplexValue(aPrefName, GB_STRING_FLAG);
				}
				catch (e) {
					var string = new GB_STRING_CONSTRUCTOR;
        			string.data = aPrefValue.join(GB_PREF_SEP);
        			GB_PREF_OBJ.setComplexValue(aPrefName, GB_STRING_FLAG, string);
				}
				
				this.mTypesTable[aPrefName] = GB_PREF_TYPE_ARRAY;
				break;

			case GB_PREF_TYPE_BOOL:
				try {
					GB_PREF_OBJ.getBoolPref(aPrefName);
				}
				catch (e) {
					GB_PREF_OBJ.setBoolPref(aPrefName, aPrefValue);
				}

				this.mTypesTable[aPrefName] = GB_PREF_TYPE_BOOL;
				break;
				
			case GB_PREF_TYPE_CHAR:
				try {
					GB_PREF_OBJ.getCharPref(aPrefName);
				}
				catch (e) {
					GB_PREF_OBJ.setCharPref(aPrefName, aPrefValue);
				}

				this.mTypesTable[aPrefName] = GB_PREF_TYPE_CHAR;
				break;
				
			case GB_PREF_TYPE_INT:
				try {
					GB_PREF_OBJ.getIntPref(aPrefName);
				}
				catch (e) {
					GB_PREF_OBJ.setIntPref(aPrefName, aPrefValue);
				}

				this.mTypesTable[aPrefName] = GB_PREF_TYPE_INT;
				break;
				
			default:
				myGooglebarUtil.logMessage("Unknown pref type (" + aPrefType + "), ignoring value for " + aPrefName);
				break;
		}
	},

	getPrefType: function(aPrefName) {
		return this.mTypesTable[aPrefName];
	},

	getPrefValue: function(aPrefName) {
		var prefType = this.getPrefType(aPrefName);
		
		if (!prefType) {
			myGooglebarUtil.logMessage("No type available for pref " + aPrefName + ", this is likely an initialization error");
			return null;
		}
	
		switch (prefType) {

			case GB_PREF_TYPE_ARRAY:
				try {
				    var prefArray = new Array(0);
				    
					try {
					  prefArray = GB_PREF_OBJ.getComplexValue(aPrefName, GB_STRING_FLAG).data.split(GB_PREF_SEP);
					}
					catch(e) {
					    myGooglebarUtil.logMessage("\nPref array init failure caught\n" + e);
					}
					
					if (prefArray.length == 1 && prefArray[0] == "") {
						prefArray = new Array(0);
					}
				
					return prefArray;
				}
				catch (e) {
					myGooglebarUtil.logMessage("Unable to retrieve updated pref " + aPrefName + " - " + e);
				}
				break;

			case GB_PREF_TYPE_BOOL:
				try {
					return GB_PREF_OBJ.getBoolPref(aPrefName);
				}
				catch (e) {
					myGooglebarUtil.logMessage("Unable to retrieve updated pref " + aPrefName + " - " + e);
				}
				break;
				
			case GB_PREF_TYPE_CHAR:
				try {
					return GB_PREF_OBJ.getCharPref(aPrefName);
				}
				catch (e) {
					myGooglebarUtil.logMessage("Unable to retrieve updated pref " + aPrefName + " - " + e);
				}
				break;
				
			case GB_PREF_TYPE_INT:
				try {
					return GB_PREF_OBJ.getIntPref(aPrefName);
				}
				catch (e) {
					myGooglebarUtil.logMessage("Unable to retrieve updated pref " + aPrefName + " - " + e);
				}
				break;
				
			default:
				myGooglebarUtil.logMessage("Unknown pref type (" + aPrefType + "), ignoring value for " + aPrefName);
				break;
		}
		
		return null;
	},

	setPrefValue: function(aPrefName, aPrefValue) {
		var prefType = this.getPrefType(aPrefName);
		
		if (!prefType) {
			myGooglebarUtil.logMessage("No type available for pref " + aPrefName + ", this is likely an initialization error");
			return;
		}
	
		switch (prefType) {

			case GB_PREF_TYPE_ARRAY:
				var string = new GB_STRING_CONSTRUCTOR;
    			string.data = aPrefValue.join(GB_PREF_SEP);

				try {
					GB_PREF_OBJ.setComplexValue(aPrefName, GB_STRING_FLAG, string);
				}
				catch (e) {
					myGooglebarUtil.logMessage("Unable to set pref " + aPrefName + " - " + e);
				}
				break;

			case GB_PREF_TYPE_BOOL:
				try {
					GB_PREF_OBJ.setBoolPref(aPrefName, aPrefValue);
				}
				catch (e) {
					myGooglebarUtil.logMessage("Unable to set pref " + aPrefName + " - " + e);
				}
				break;
				
			case GB_PREF_TYPE_CHAR:
				try {
					GB_PREF_OBJ.setCharPref(aPrefName, aPrefValue);
				}
				catch (e) {
					myGooglebarUtil.logMessage("Unable to set pref " + aPrefName + " - " + e);
				}
				break;
				
			case GB_PREF_TYPE_INT:
				try {
					GB_PREF_OBJ.setIntPref(aPrefName, aPrefValue);
				}
				catch (e) {
					myGooglebarUtil.logMessage("Unable to set pref " + aPrefName + " - " + e);
				}
				break;
				
			default:
				myGooglebarUtil.logMessage("Unknown pref type (" + aPrefType + "), ignoring value for " + aPrefName);
				break;
		}
	}
};

myGooglebarPrefUtil.initialize();